# Imports from us-elections.
from elections.utils.builts import ChamberFilterableList


class Party(object):
    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            self.__dict__[k] = v
        self.held_seats = ChamberFilterableList()

    def add_held_seat(self, seat):
        self.held_seats.append(seat)

    @property
    def holds_seats(self):
        return len(self.held_seats) > 0

    def __repr__(self):
        return "<Party: {}>".format(self.__str__())

    def __str__(self):
        return self.name



def ordinalize(cardinal):
    cardinal = str(cardinal)
    if cardinal.endswith("1") and not cardinal.endswith("11"):
        cardinal += "st"
    elif cardinal.endswith("2") and not cardinal.endswith("12"):
        cardinal += "nd"
    elif cardinal.endswith("3") and not cardinal.endswith("13"):
        cardinal += "rd"
    else:
        cardinal += "th"
    return cardinal



# Imports from us-elections.
from elections.models.governments.constants import BRANCHES
from elections.models.governments.constants import EXECUTIVE
from elections.models.governments.constants import JUDICIAL
from elections.models.governments.constants import LEGISLATIVE
from elections.models.governments.government_branch import ExecutiveBranch
from elections.models.governments.government_branch import GovernmentBranch
from elections.models.governments.government_level import FederalGovernment
from elections.models.governments.government_level import GovernmentLevel
from elections.models.governments.government_level import StateGovernment


__all__ = [
    "BRANCHES",
    "EXECUTIVE",
    "ExecutiveBranch",
    "FederalGovernment",
    "GovernmentBranch",
    "GovernmentLevel",
    "JUDICIAL",
    "LEGISLATIVE",
    "StateGovernment",
]



# Imports from us-elections.
from elections.models.electoral_votes.zones import DistrictElectoralZone
from elections.models.electoral_votes.zones import StateElectoralZone


__all__ = ["DistrictElectoralZone" "StateElectoralZone"]



# Imports from other dependencies.
import us
from us.states import State as StateKlass


# Imports from us-elections.
from elections.models.governments.constants import BRANCHES
from elections.utils.builts import ChamberFilterableList


class GovernmentBranch(object):
    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            if k == "branch_type" and v in BRANCHES:
                self.type = v
            elif k == "seats":
                if v:
                    self.__dict__[k] = v
            else:
                self.__dict__[k] = v

    @property
    def jurisdiction_name(self):
        return self.jurisdiction.__dict__["name"]

    @property
    def jurisdiction_abbreviation(self):
        return self.jurisdiction.__dict__["ap_abbr"]

    @property
    def verbose_name(self):
        if hasattr(self, "body_name"):
            if self.jurisdiction_abbreviation == "U.S.":
                return "{} {}".format(
                    self.jurisdiction_abbreviation, self.body_name
                )
            return "{} {}".format(self.jurisdiction_name, self.body_name)
        return self.jurisdiction_name

    @property
    def is_empty(self):
        return not hasattr(self, "seats")

    def count_seats(self):
        if self.is_empty:
            return 0
        else:
            return len(self.seats)

    def __repr__(self):
        return "<{}Branch: {}>".format(
            self.type.capitalize(), self.verbose_name
        )

    def __str__(self):
        if self.verbose_name != self.jurisdiction_name:
            return "{} ({} {} Branch)".format(
                self.verbose_name,
                self.jurisdiction_abbreviation,
                self.type.capitalize(),
            )
        return "{} {} Branch".format(
            self.jurisdiction_abbreviation, self.type.capitalize()
        )


class LegislativeBranch(GovernmentBranch):
    def __init__(self, **kwargs):
        super(LegislativeBranch, self).__init__(**kwargs)

        if not isinstance(self.jurisdiction, StateKlass):

            def get_seats_for_state(state):
                state = us.states.lookup(state)
                return ChamberFilterableList(
                    [seat for seat in self.seats if seat.state == state]
                )

            self.seats_for_state = get_seats_for_state


class ExecutiveBranch(GovernmentBranch):
    @property
    def chief(self):
        if self.count_seats():
            return self.seats.chief
        return None



# Imports from python.
import os
from setuptools import find_packages
from setuptools import setup


# Imports from us-elections.
from elections import __appname__, __version__


REPO_URL = "https://github.com/The-Politico/us-elections/"

PYPI_VERSION = ".".join(str(v) for v in __version__)

with open(os.path.join(os.path.dirname(__file__), "README.md")) as readme:
    README = readme.read()

setup(
    name=__appname__,
    version=PYPI_VERSION,
    packages=find_packages(exclude=["db", "docs", "tests"]),
    license="MIT",
    description="U.S. election and government metadata",
    long_description=README,
    long_description_content_type="text/markdown",
    url=REPO_URL,
    download_url="{repo_url}archive/{version}.tar.gz".format(
        **{"repo_url": REPO_URL, "version": PYPI_VERSION}
    ),
    author="POLITICO Interactive News",
    author_email="interactives@politico.com",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Internet :: WWW/HTTP",
    ],
    keywords="",
    platforms=["any"],
    package_data={"elections": ["data/**/*.pkl", "data/*.pkl"]},
    include_package_data=True,
    install_requires=["us"],
    zip_safe=False,
    extras_require={"test": ["pytest"]},
)
